<?php

namespace Autopilot\Controller;

use App\Controller\App;


class Autopilot extends App {


    public function image() {

        $this->helper('session')->close();
        $this->hasValidCsrfToken(true);

        $prompt = $this->param('prompt');
        $size = $this->param('size');
        $style = $this->param('style', 'natural');

        if (!$prompt || !$size) {
            return false;
        }

        if (function_exists('ini_set')) {
            ini_set('max_execution_time', 600);
            ini_set('output_buffering', 'Off');
        }

        header('Content-Type: text/plain');
        header('Cache-Control: no-cache');
        header('Connection: keep-alive');

        $response = $this->module('autopilot')->client()->images()->create([
            'model' => $this->app->retrieve('autopilot/openAI/image-model', 'dall-e-3'),
            'prompt' => $prompt,
            'n' => 1,
            'style' => $style,
            'size' => $size,
            'response_format' => 'url',
        ]);

        return ['url' => $response->toArray()['data'][0]['url'] ?? null];
    }

    public function chat() {

        $this->helper('session')->close();
        $this->hasValidCsrfToken(true);

        $messages = $this->param('messages', []);

        $uid = $this->param('uid');

        if (!is_array($messages) || !count($messages)) {
            return false;
        }

        if (ob_get_level()) ob_end_clean();

        if (function_exists('ini_set')) {
            ini_set('max_execution_time', 600);
            ini_set('output_buffering', 'Off');
        }

        header('Content-Type: text/plain');
        header('Cache-Control: no-cache');
        header('Connection: keep-alive');

        $stream = $this->module('autopilot')->client()->chat()->createStreamed([
            'model' => $this->app->retrieve('autopilot/openAI/model', 'gpt-3.5-turbo'),
            'messages' => $messages,
        ]);

        $content = '';
        $chunk = 0;

        ob_start();

        foreach ($stream as $response) {

            $chunk += 1;
            $text = $response->choices[0]->delta->content;

            $content .= $text;
            echo $text;

            if ($chunk >= 50) {

                if ($uid) {
                    $this->app->memory->set($uid, $content);
                }

                $chunk = 0;
                ob_flush();
                flush();
            }
        }

        if ($uid) {
            $this->app->memory->del($uid);
        }

        return '';
    }

    public function progress($uid = null) {

        $this->helper('session')->close();
        $this->hasValidCsrfToken(true);

        $uid = $this->param('uid', $uid);

        if (!$uid) {
            return false;
        }

        return ['content' => $this->app->memory->get($uid)];
    }

    public function imageToAsset() {

        $this->helper('session')->close();
        $this->hasValidCsrfToken(true);

        $url = $this->param('url');

        if (!$url) {
            return false;
        }

        $ext = strtolower(pathinfo(parse_url($url, PHP_URL_PATH), PATHINFO_EXTENSION));
        $tmp = $this->app->path('#tmp:').'/'.uniqid().".{$ext}";

        // Fetch the file from the external server
        $contents = @file_get_contents($url);

        if ($contents !== false) {

            $saveFile = file_put_contents($tmp, $contents);

            if ($saveFile !== false) {

                $meta = ['folder' => $this->param('folder', '')];

                $param = [
                    'name' => ["Autopilot-image.{$ext}"],
                    'full_path' => ["Autopilot-image.{$ext}"],
                    'type' => ["image/{$ext}"],
                    'tmp_name' => [$tmp],
                    'error' => [0],
                    'size' => [filesize($tmp)],
                ];

                return [
                    'asset' => $this->module('assets')->upload($param, $meta, false)['assets'][0] ?? null,
                ];
            }
        }

        throw new \Exception('Failed to import generated image.');
    }

}
