const actions = {

    prompt: {
        label: 'Prompt',
        icon: 'chat',
        handler(root) {

            VueView.ui.modal('autopilot:assets/dialogs/prompt.js', {
                fields: Object.keys(root.document).filter(field => field[0] !== '_'),
            }, {
                onSubmit: (prompt) => {

                    if (!prompt.trim()) return;

                    let message = [
                        'Use the following JSON Object as reference if fields are mentioned in the Objective:',
                        JSON.stringify(root.document),
                        '',
                        'System:',
                        'Use the user input to generate content. Try to recognize references in the user input (if possible) to the fields and its values in the JSON Object.',
                        'Return always the result or suggestion without any describing information.',
                        'Response needs to be in HTML format.',
                        '',
                        'User input:',
                        prompt
                    ].join("\n");

                    root.execute([{
                        role: 'user',
                        content: message
                    }], (msg) => {
                        root.updateValue(msg);
                    });
                }
            }, {
                size: 'large',
                escape: true
            });

        }
    },

    translate: {
        label: 'Translate',
        handler(root) {

            if (!root.val || !root.val.trim()) {
                return;
            }

            let to = '';

            if (root.field.i18n && root.locale && root.locale !== 'default') {
                to = root.locale;
            }

            App.ui.prompt('Translate to', to, (locale) => {

                if (!locale.trim()) return;

                let message = [
                    'System:',
                    'Response needs to be in HTML format.',
                    'Return always the result or suggestion without any describing information.',
                    '',
                    `Target Language: ${locale}`,
                    '',
                    'Text for translation:',
                    root.val
                ].join("\n");

                root.execute([{
                    role: 'user',
                    content: message
                }], (msg) => {
                    root.updateValue(msg);
                });
            });
        }
    },

    shorten: {
        label: 'Shorten',
        handler(root) {

            if (!root.val || !root.val.trim()) {
                return;
            }

            App.ui.prompt('Max length', root.val.length, (maxlength) => {

                if (!maxlength || !(!isNaN(parseFloat(maxlength)) && isFinite(maxlength)) || root.val.length <= maxlength) {
                    return;
                }

                let message = [
                    'System:',
                    'Response needs to be in HTML format.',
                    'Return always the result or suggestion without any describing information.',
                    '',
                    `Shorten the following text (max length: ${maxlength}):`,
                    root.val
                ].join("\n");

                root.execute([{
                    role: 'user',
                    content: message
                }], (msg) => {
                    root.updateValue(msg);
                });
            });

        }
    }
}

export default {

    init() {
        return new Promise((resolve, reject) => {
            resolve(actions);
        });
    },
}
