<?php

/**
 *
 * @OA\Tag(
 *   name="detektivo",
 *   description="Detektivo module",
 * )
 */

$this->on('restApi.config', function($restApi) {

    /**
     * @OA\Get(
     *     path="/detektivo/search/{index}",
     *     tags={"detektivo"},
     *     @OA\Parameter(
     *         description="Index name",
     *         in="path",
     *         name="index",
     *         required=true,
     *         @OA\Schema(type="string")
     *     ),
     *    @OA\Parameter(
     *         description="Query string",
     *         in="query",
     *         name="q",
     *         required=true,
     *         @OA\Schema(type="String")
     *     ),
     *    @OA\Parameter(
     *         description="Maximum number of documents returned",
     *         in="query",
     *         name="limit",
     *         required=false,
     *         @OA\Schema(type="Integer")
     *     ),
     *    @OA\Parameter(
     *         description="Number of documents to skip",
     *         in="query",
     *         name="offset",
     *         required=false,
     *         @OA\Schema(type="Integer")
     *     ),
     *     @OA\Response(response="200", description="List of matched documents"),
     *     @OA\Response(response="404", description="Index not found")
     * )
    */

    $restApi->addEndPoint('/detektivo/search/{index}', [

        'GET' => function($params, $app) {

            $index = $params['index'];

            if (!$app->module('detektivo')->exists($index)) {
                return false;
            }

            $idx = $app->module('detektivo')->index($index);
            $query = $app->param('q:string', '');
            $filter = $app->param('filter', null);
            $limit = $app->param('limit:int', 50);
            $offset = $app->param('limit:int', 0);

            if (!trim($query)) {
                $app->response->status = 412;
                return ['error' => 'Query parameter is missing or empty!'];
            }

            $params = [
                'limit' => $limit,
                'offset' => $offset,
            ];

            if ($filter) {
                $params['filter'] = $filter;
            }

            return $idx->search($query, $params);
        }
    ]);

});

$this->on('graphql.config', function($gql) {
    $app = $this;
    include(__DIR__.'/graphql/detektivo.php');
});
