export default {

    _meta: {
        label: 'Page Link / Url',
        info: 'Url or page link',
        icon: 'pages:assets/icons/link.svg',
        settings: [
            {name: 'link', type: 'text'},
        ],
        render(value, field, context) {
            return value;
        }
    },

    data() {
        return {
            page: null,
            val: this.modelValue
        }
    },

    props: {
        modelValue: {
            type: String,
            default: ''
        },
        placeholder: {
            type: String,
            default: 'https://...'
        },
        locale: {
            type: String,
            default: null
        }
    },

    mounted() {
        this.loadPage();
    },

    computed: {
        hasLinkedPage() {
            return this.page && this.val && this.val.indexOf(this.page._id) !== -1;
        }
    },

    watch: {
        modelValue() {
            this.val = this.modelValue;
            this.update();
        }
    },

    methods: {

        selectPage() {

            VueView.ui.offcanvas('pages:assets/dialogs/page-picker.js', {page: this.val, locale: this.locale}, {

                select: (page, locale) => {
                    this.page = page;
                    this.val = `pages://${page._id}`+((locale && locale != 'default' && `?locale=${locale}`) || '');
                    this.update();
                }

            }, {flip: true, size: 'large'})

        },

        loadPage() {

            if (!this.val || !this.val.match(/^pages:\/\//)) {
                return;
            }

            const url = new URL(this.val), locale = url.searchParams.get('locale');

            let options = {
                filter: {_id: url.pathname.substr(2)},
                limit: 1
            };

            if (locale && App._locales[locale]) {
                options.locale = locale;
            }

            this.$request('/pages/utils/find', {options}).then(rsp => {

                this.page = null;

                if (rsp.items && rsp.items[0]) {
                    this.page = rsp.items[0];
                }
            }).catch(res => {
                App.ui.notify(res.error || 'Loading page failed!', 'error');
            });
        },

        update() {
            this.$emit('update:modelValue', this.val)
        }
    },

    template: /*html*/`
        <div field="page-url">
            <kiss-card class="kiss-padding-small kiss-size-small kiss-position-relative kiss-margin-small" v-if="hasLinkedPage" theme="bordered contrast">
                <div class="kiss-text-bold">{{ page.title }}</div>
                <div class="kiss-color-muted kiss-size-xsmall kiss-text-truncate">{{page._r}}</div>
                <a class="kiss-cover" :href="$route('/pages/page/'+page._id)" target="_blank"></a>
            </kiss-card>
            <div class="kiss-flex">
                <input type="text" class="kiss-input kiss-flex-1 kiss-margin-small-right" :class="{'kiss-color-muted': hasLinkedPage}" v-model="val" @input="update" :placeholder="placeholder">
                <button class="kiss-button" type="button" @click="selectPage">Page</button>
            </div>
        </div>
    `,
}
